;
;*******************************************************
;*******************************************************
;**                                                   **
;**    Sztuka programowania mikrokontrolerow AVR      **
;**                                                   **
;**         Program obslugi klawiatury                **
;**                  wariant 3                        **
;**                                                   **
;**       Andrzej Pawluczuk                           **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                Bialystok, 2006                    **
;**                                                   **
;*******************************************************
;*******************************************************
;
;*******************************************************
	.nolist
	.include "m8515def.inc"
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
;
;*******************************************************
;*******************************************************
;**                                                   **
;**   Uzywane w programie rejestry                    **
;**   Znaczenie poszczegolnych bitow                  **
;**                                                   **
;*******************************************************
;*******************************************************
;
.equ	Stand_TIMSK	=	0b00000010
;	Timer/counter		  ||| | ||
;	Interrupt Mask		  ||| | |+-- OCIE0 : Timer/Counter0 Output Compare Match Interrupt Enable
;				  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;				  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;				  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;				  |+-------- OCIE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;				  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ	Stand_TCCR0	=	0b00000100
;	Timer/Counter0 Control	  ||||||||
;				  |||||||+-- CS00 : Clock Select
;				  ||||||+--- CS01 : Clock Select
;				  |||||+---- CS02 : Clock Select
;				  ||||+----- WGM01 : Waveform Generation Mode
;				  |||+------ COM00 : Compare Match Output Mode
;				  ||+------- COM01 : Compare Match Output Mode
;				  |+-------- WGM00 : Waveform Generation Mode
;				  +--------- FOC0 : Force Output Compare
;
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.def	acc		= r16
.def	acc2		= r17
.def	param		= r18
;-----------------------------------------------------------------------------
.equ	KeyPort		= PORTD
.equ	LEDPort		= PORTB
.equ	KeyPDirection	= KeyPort - 1
.equ	LedPDirection	= LEDPort - 1
.equ	KeyPInput	= KeyPort - 2
;-----------------------------------------------------------------------------
.equ	Key0Pin		= 0
.equ	Key1Pin		= 1
.equ	Key2Pin		= 2
.equ	Key3Pin		= 3
.equ	Key4Pin		= 4
.equ	Key5Pin		= 5
.equ	Key6Pin		= 6
.equ	Key7Pin		= 7
;-----------------------------------------------------------------------------
.equ	Key0Code	= 0x01
.equ	Key1Code	= 0x02
.equ	Key2Code	= 0x03
.equ	Key3Code	= 0x04
.equ	Key4Code	= 0x05
.equ	Key5Code	= 0x06
.equ	Key6Code	= 0x07
.equ	Key7Code	= 0x08
.equ	Key8Code	= 0x09
.equ	Key9Code	= 0x0A
.equ	Key10Code	= 0x0B
.equ	Key11Code	= 0x0C
.equ	Key12Code	= 0x0D
.equ	Key13Code	= 0x0E
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;USART Receive Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;USART Data Register Empty Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;USART Transmit Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ldi	acc,0xFF			; [DDRA] := <output> ;
	out	DDRA,acc			;
	out	DDRB,acc			; [DDRB] := <output> ;
	out	DDRC,acc			; [DDRC] := <output> ;
	out	DDRD,acc			; [DDRD] := <output> ;
	out	PORTA,acc			; [PORTA] := 0xFF ;
	out	PORTB,acc			; [PORTB] := 0xFF ;
	out	PORTC,acc			; [PORTC] := 0xFF ;
	out	PORTD,acc			; [PORTD] := 0xFF ;
	ret					;END HardwareInit ;
;-----------------------------------------------------------------------------
.include "keybsrv.inc"
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,0				; KeyPort := <input> + <pullup>;
	out	KeyPDirection,acc		;
	ldi	acc,0xFF			;
	out	KeyPort,acc			;
	out	LedPDirection,acc		; LEDPort := <output> + <outp.=high> ;
	out	LEDPort,acc			;
	ret					;END EnvirInit ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	rcall	KeybSoftInit			; KeybSoftInit ( ) ;
	ret					;END SoftwareInit ;
;-----------------------------------------------------------------------------
AnyKeyActive :					;FUNCTION AnyKeyActive ( ) : BOOLEAN [C] ;
;*************					;BEGIN (* AnyKeyActive *)
	in	acc,KeyPInput			; IF ( KeyPInput AND 0x7F ) = 0x7F THEN
	andi	acc,0x7F			;
	cpi	acc,0x7F			;
	brne	AnKeAct_0			; BEGIN
	clc					;  AnyKeyActive := FALSE ;
	ret					;  RETURN ;
AnKeAct_0 :					; END (* IF *) ;
	sec					; AnyKeyActive := TRUE ;
	ret					;END (* AnyKeyActive *) ;
;-----------------------------------------------------------------------------
DecodeKey :					;PROCEDURE DecodeKey ( ) ;
;**********					;VAR
;						; Kbd [acc2] : BYTE ;
;						; Temp [acc] : BYTE ;
						;BEGIN (* DecodeKey *)
	in	acc2,KeyPInput			; Kbd := KeyPInput ;
	ldi	acc,0				; Temp := 0 ;
	sbrs	acc2,Key7Pin			; IF Kbd[Key7Pin] THEN (* if shift *)
	rjmp	DecoKe_0			; BEGIN
	sbrs	acc2,Key0Pin			;  IF Kbd[Key0Pin] THEN
	ldi	acc,Key0Code			;   Temp := Key0Code ;
	sbrs	acc2,Key1Pin			;  IF Kbd[Key1Pin] THEN
	ldi	acc,Key1Code			;   Temp := Key1Code ;
	sbrs	acc2,Key2Pin			;  IF Kbd[Key2Pin] THEN
	ldi	acc,Key2Code			;   Temp := Key2Code ;
	sbrs	acc2,Key3Pin			;  IF Kbd[Key3Pin] THEN
	ldi	acc,Key3Code			;   Temp := Key3Code ;
	sbrs	acc2,Key4Pin			;  IF Kbd[Key4Pin] THEN
	ldi	acc,Key4Code			;   Temp := Key4Code ;
	sbrs	acc2,Key5Pin			;  IF Kbd[Key5Pin] THEN
	ldi	acc,Key5Code			;   Temp := Key5Code ;
	sbrs	acc2,Key6Pin			;  IF Kbd[Key6Pin] THEN
	ldi	acc,Key6Code			;   Temp := Key6Code ;
	rjmp	DecoKe_1			; END (* IF ... *)
						; ELSE
DecoKe_0 :					; BEGIN
	sbrs	acc2,Key0Pin			;  IF Kbd[Key0Pin] THEN
	ldi	acc,Key7Code			;   Temp := Key7Code ;
	sbrs	acc2,Key1Pin			;  IF Kbd[Key1Pin] THEN
	ldi	acc,Key8Code			;   Temp := Key8Code ;
	sbrs	acc2,Key2Pin			;  IF Kbd[Key2Pin] THEN
	ldi	acc,Key9Code			;   Temp := Key9Code ;
	sbrs	acc2,Key3Pin			;  IF Kbd[Key3Pin] THEN
	ldi	acc,Key10Code			;   Temp := Key10Code ;
	sbrs	acc2,Key4Pin			;  IF Kbd[Key4Pin] THEN
	ldi	acc,Key11Code			;   Temp := Key11Code ;
	sbrs	acc2,Key5Pin			;  IF Kbd[Key5Pin] THEN
	ldi	acc,Key12Code			;   Temp := Key12Code ;
	sbrs	acc2,Key6Pin			;  IF Kbd[Key6Pin] THEN
	ldi	acc,Key13Code			;   Temp := Key13Code ;
DecoKe_1 :					; END (* IF ... ELSE *)
	rcall	StoreKey			; StoreKey ( Temp ) ;
	ret					;END (* DecodeKey *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	rcall	KeybService			; KeybService ( ) ;
	popz					;
	popf					;
	reti					;END TC0OvfInterrupt ;
;-----------------------------------------------------------------------------
ClearRegs	:				;PROCEDURE ClearRegs ;
;****************				;BEGIN (* ClearRegs *)
	clr	R0				;
	ldz	0x1D				;
ClrR_0	:					;
	st	Z,R0				;
	dec	Zl				;
	brne	ClrR_0				;
	ret					;END ClearRegs ;
;-----------------------------------------------------------------------------
ClearRAM :					;PROCEDURE ClearRAM ;
;*********					;BEGIN (* ClearRAM *)
	ldz	RAMEND-1			; Z := ADR ( RAMEND ) - 1 ;
	clr	acc				;
CleRA_0	:					; LOOP
	st	-Z,acc				;  Z := Z - 1 ;
						;  Z ^ := 0 ;
	cpi	ZH,0				;  IF Z = 0x60 THEN
	brne	CleRA_2				;
	cpi	ZL,0x60				;
	brne	CleRA_2				;  BEGIN
	rjmp	CleRA_1				;   EXIT ;
CleRA_2	:					;  END (* IF *) ;
	rjmp	CleRA_0				;
CleRA_1	:					; END (* LOOP *) ;
	ret					;END ClearRAM ;
;-----------------------------------------------------------------------------
RunCommand :					;PROCEDURE RunCommand ( Ch [param] : CHAR ) ;
;***********					;BEGIN
	com	param				; LEDPort := ~ Ch ;
	out	LEDPort,param			;
	ret					;END (* RunCommand *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
Main_0	:					; LOOP
	rcall	KeyPressed			;  IF KeyPressed ( ) THEN
	brcc	Main_1				;  BEGIN
	rcall	ReadKey				;   RunCommand ( ReadKey ( ) ) ;
	mov	param,acc			;
	rcall	RunCommand			;
Main_1	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END Main ;
;-----------------------------------------------------------------------------
.exit
