;
;*******************************************************
;*******************************************************
;**                                                   **
;**    Sztuka programowania mikrokontrolerow AVR      **
;**                                                   **
;**      Program prezentacji przerwan od czasu        **
;**                                                   **
;**       Andrzej Pawluczuk                           **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                Bialystok, 2006                    **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def	acc		= r16
;-----------------------------------------------------------------------------
.equ	StatusLED	=	0
.equ	DelayLed	=	1
.equ	TimerData	=	0x70
.equ	DivideConst	=	7
.equ	DelayTime	=	32
.equ	KeyPin		=	7
;-----------------------------------------------------------------------------
.equ	Stand_TIMSK	=	0b00000010
.equ	Stand_TCCR0	=	0b00000101
.equ	Stand_PADir	=	0b00000000
.equ	Stand_PAData	=	0b11111111
.equ	Stand_PBDir	=	0b11111111
.equ	Stand_PBData	=	0b11111111
.equ	Stand_PCDir	=	0b11111111
.equ	Stand_PCData	=	0b11111111
.equ	Stand_PDDir	=	0b11111111
.equ	Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg					;VAR
TimeDivider :		.byte	1		; TimeDivider : BYTE ;
Status :		.byte	1		; Status : BYTE ;
DelayCnt :		.byte	1		; DelayCnt : BYTE ;
TimeUp :		.byte	1		; TimeUp : BOOLEAN ;
;-----------------------------------------------------------------------------
	.cseg
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;USART Receive Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;USART Data Register Empty Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;USART Transmit Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit	:				;PROCEDURE SoftwareInit ;
;****************				;BEGIN (* SoftwareInit *)
	ldi	acc,0				; TimeDivider := 0 ;
	sts	TimeDivider,acc			;
	sts	Status,acc			; Status := 0 ;
	sts	DelayCnt,acc			; DelayCnt := 0 ;
	sts	TimeUp,acc			; TimeUp := FALSE ;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	ldz	TimeDivider			; TimeDivider := TimeDivider + 1 ;
	ld	acc,Z				;
	inc	acc				;
	st	Z,acc				;
	cpi	acc,DivideConst			; IF TimeDivider = DivideConst THEN
	brne	TC0OV_0				; BEGIN
	ldi	acc,0				;  TimeDivider := 0 ;
	st	Z,acc				;
	ldz	Status				;  IF Status = 0 THEN
	ld	acc,Z				;
	cpi	acc,0				;
	brne	TCO0V_1				;  BEGIN
	ldi	acc,1				;   Status := 1 ;
	st	Z,acc				;
	sbi	PORTB,StatusLED			;   PORTB [ StatusLED ] := 1 ;
	rjmp	TCO0V_2				;  END (* IF ... *)
TCO0V_1	:					;  ELSE
						;  BEGIN
	ldi	acc,0				;   Status := 0 ;
	st	z,acc				;
	cbi	PORTB,StatusLED			;   PORTB [ StatusLED ] := 0 ;
TCO0V_2	:					;  END (* IF ... ELSE *) ;
TC0OV_0	:					; END (* IF *) ;
	ldz	DelayCnt			; IF DelayCnt <> 0 THEN
	ld	acc,Z				;
	cpi	acc,0				;
	breq	TC0OV_3				; BEGIN
	dec	acc				;  DelayCnt := DelayCnt - 1 ;
	st	Z,acc				;
	cpi	acc,0				;  IF DelayCnt = 0 THEN
	brne	TC0OV_4				;  BEGIN
	ldi	acc,1				;   TimeUp := TRUE ;
	sts	TimeUp,acc			;
TC0OV_4	:					;  END (* IF *) ;
TC0OV_3	:					; END (* IF *) ;
	ldi	acc,TimerData			; TCNT0 := TimerData ;
	out	TCNT0,acc			;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
Main_0	:					; LOOP
	in	acc,PINA			;  IF PINA [ KeyPin ] = 0 THEN
	andi	acc,1<<KeyPin			;
	brne	Main_1				;  BEGIN
	cbi	PORTB,DelayLed			;   PORTB [ DelayLed ] := 0 ;
	ldi	acc,DelayTime			;   DelayCnt := DelayTime ;
	sts	DelayCnt,acc			;
	ldi	acc,0				;   TimeUp := FALSE ;
	sts	TimeUp,acc			;
Main_2	:					;   REPEAT
	lds	acc,TimeUp			;   UNTIL TimeUp ;
	cpi	acc,0				;
	breq	Main_2				;
	sbi	PORTB,DelayLed			;   PORTB [ DelayLed ] := 1 ;
Main_3	:					;   REPEAT
	in	acc,PINA			;   UNTIL PINA [ KeyPin ] <> 0 ;
	andi	acc,1<<KeyPin			;
	breq	Main_3				;
Main_1	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
.exit
